<?php

/**
 * @copyright Copyright 2023 Genius Plugin, Corp. © All Rights Reserved
 * @version 1.0.0
 * @author geniusplugin.com
 */
if (!defined("WHMCS")) {
    die("This file cannot be accessed directly");
}

use WHMCS\Database\Capsule;




require_once 'lib/towers.php';

/**
 * Related abilities and settings.
 * @return array
 */
function gpf_MetaData()
{
    return array(
        'DisplayName' => 'Genius Plugin Firewall',
        'APIVersion' => '1.0.0',
        'RequiresServer' => true,
        'DefaultNonSSLPort' => '80',
        'DefaultSSLPort' => '443',

    );
}


/**
 * Additional actions for admin
 * @return array
 */
function gpf_AdminCustomButtonArray()
{
?>
    <style>
        #servicecontent input,
        #servicecontent .form-control {
            border: 1px solid burlywood !important;
        }

        .btn-primary,
        table.datatable th {
            color: #000 !important;
            border-radius: 5px !important;
            background: burlywood !important;
            border: 1px solid cadetblue !important;
            background-color: burlywood !important;
        }

        #servicecontent .btn {
            color: #000 !important;
            background-color: #efefef;
            border: 1px solid burlywood !important;
        }

        table.form td.fieldarea {
            opacity: 0.95;
        }

        #servicecontent .form {
            background-image: url('/../modules/servers/gpf/logo.svg');
            background-repeat: no-repeat;
            background-position: 100% 0%;
            background-size: 70vh auto;
        }

        @media (max-width: 640px) {
            #servicecontent .form {
                background-image: url('/../modules/servers/gpf/logo-w.svg');
            }
        }

        #btnLoginLinkTrigger {
            display: none;
        }
    </style>
<?php
    return array(
        "Check Credit" => "checkCredit",
        "Status" => "Status",
        "Login to Dashboard" => "LoginLink"
    );
}


function curlRequest($endpoint, $params, $method = 'GET', $data = [])
{
    $data = http_build_query($data);
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, 'https://' . $params['serverhostname'] . '/api/v1/' . $endpoint . '/');
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
    curl_setopt($ch, CURLOPT_MAXREDIRS, 3);
    curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 3);
    curl_setopt($ch, CURLOPT_TIMEOUT, 20);
    switch ($method) {
        case 'GET':
            break;
        case 'POST':
            curl_setopt($ch, CURLOPT_POST, true);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
            break;
        case 'PUT':
            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'PUT');
            curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
            break;
        case 'DELETE':
            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'DELETE');
            break;
    }

    $response = curl_exec($ch);
    if (curl_errno($ch)) {
        logModuleCall('gpf', $endpoint, $params, curl_error($ch), '', '');
        curl_close($ch);
        // Return the error response
        return json_encode(array('status' => 'error: ' . curl_error($ch)));
    }
    curl_close($ch);

    return $response;
}

/**
 * Test connection with the given server parameters.
 * @param array $params common module parameters
 * @return array
 */
function gpf_TestConnection(array $params)
{

    try {
        // Call the service's connection test function.
        $key = sha1(trim($params['serveraccesshash']));
        $timestamp = str_shuffle(time());
        $domain = 'example.com';
        $hash = sha1($key . $domain . $timestamp);
        $endpoint = 'connection';
        $data = [
            'domain' => $domain,
            'hash' => $hash,
            'timestamp' => $timestamp

        ];
        $response = json_decode(curlRequest($endpoint, $params, 'POST', $data));
        if (substr($response->status, 0, 2) != 'OK') {
            $success = false;
            if (is_object($response)) {
                $errorMsg = $response->status;
            } else {
                $errorMsg = $response;
            }
        } else {
            $success = true;
            $errorMsg = '';
        }
    } catch (Exception $e) {
        logModuleCall(
            'gpf',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        $success = false;
        $errorMsg = $e->getMessage();
    }
    if ($success === false && $errorMsg == null) return 'Can not get remote response.';
    return array(
        'success' => $success,
        'error' => $errorMsg,
    );
}

/**
 * Get available Credit on GP Cloud
 * @param array $params common module parameters
 * @return array
 */
function gpf_checkCredit(array $params)
{

    try {
        // Call the service's connection test function.
        $key = sha1(trim($params['serveraccesshash']));
        $timestamp = str_shuffle(time());
        $domain = 'example.com';
        $hash = sha1($key . $domain . $timestamp);
        $endpoint = 'getcredit';
        $data = [
            'domain' => $domain,
            'hash' => $hash,
            'timestamp' => $timestamp

        ];
        $response = json_decode(curlRequest($endpoint, $params, 'POST', $data));
        if (substr($response->status, 0, 2) != 'OK') {
            $success = false;
            if (is_object($response)) {
                $errorMsg = $response->status;
            } else {
                $errorMsg = $response;
            }
        } else {
            $success = true;
            $errorMsg = '';
        }
    } catch (Exception $e) {
        logModuleCall(
            'gpf',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        $success = false;
        $errorMsg = $e->getMessage();
    }

    if ($success === false && $errorMsg == null) return 'Can not get remote response.';
    if ($success === false) return array('error' => $errorMsg);
    return array('success' => $response->status);
}


/**
 * Define product configuration options.
 * This must be a valid package name provided by GP
 * @return array
 */
function gpf_ConfigOptions()
{
    return array(
        'Name on server' => array(
            'Type' => 'text',
            'Size' => '24',
            'Default' => 'Free',
            'Description' => '<br>Case sensitive. This must be a valid package name provided by GP. 
            <br>(<style>.label{text-transform:lowercase !important;font-size: 12px;} .label.active{text-transform:none !important;} .pro{background-color: #0768b8;}</style><span class="label active" style="">Free</span>, <span class="label pro">20k</span> <span class="label pro">40k</span> <span class="label pro">80k</span> <span class="label pro">200k</span> <span class="label pro">500k</span> ..etc)',
        ),

    );
}

/**
 * Find the domain name in params or database
 * Checks the addon domain, then the service domain, then the invoice details for related domain name
 * @param array $params
 * @return string
 */
function gpf_FindDomain($params)
{
    $domain_name = '';

    if (isset($params['model']['custom_field_values'])) {
        foreach ($params['model']['custom_field_values'] as $customField) {
            if (isset($customField['custom_field']['fieldname']) && $customField['custom_field']['fieldname'] === 'domain') {
                $domain_name = $customField['value'];
                return $domain_name;
            }
        }
    }
    if (isset($params["domain"]) && !empty($params["domain"])) {
        $domain_name = $params["domain"];
        return $domain_name;
    }
    if (isset($params["model"]["service"]["domain"]) && !empty($params["model"]["service"]["domain"])) {
        $domain_name = $params["model"]["service"]["domain"];
        return $domain_name;
    }

    // Get the domain the simple way
    if (isset($params["pid"]) && is_numeric($params["pid"])) {
        foreach (Capsule::table('tblcustomfields')
            ->select('id')
            ->where('relid', '=', $params["pid"])
            ->where('fieldname', '=', 'domain')
            ->orderBy('id', 'DESC')
            ->limit(1)
            ->get() as $addon_customfield) {
            $addon_custom_field_id = $addon_customfield->id;
        }

        if (!empty($addon_custom_field_id)) {
            if (isset($params["addonId"]) && is_numeric($params["addonId"])) {
                foreach (Capsule::table('tblcustomfields')
                    ->join('tblcustomfieldsvalues', 'tblcustomfieldsvalues.fieldid', '=', 'tblcustomfields.id')
                    ->select('tblcustomfields.fieldname', 'tblcustomfieldsvalues.value')
                    ->where('tblcustomfieldsvalues.relid', '=', $params["addonId"])
                    ->orderBy('tblcustomfields.fieldname', 'ASC')
                    ->get() as $addon_field) {
                    if ($addon_field->fieldname == 'domain' && !empty($addon_field->value)) {
                        $domain_name = $addon_field->value;
                        return $domain_name;
                    }
                }

                foreach (Capsule::table('tblinvoiceitems')
                    ->select('description')
                    ->where('type', '=', 'Addon')
                    ->where('relid', '=', $params["addonId"])
                    ->orderBy('id', 'DESC')
                    ->limit(1)
                    ->get()  as $addon_description_field) {
                    $addon_description = $addon_description_field->description;
                }


                if (!empty($addon_description)) {
                    if (preg_match('/\((.*?)\)/', $addon_description, $matches)) {
                        $result = $matches[1];
                        $domain_name = $params["domain"] = $params["customfields"]["domain"] = $params["model"]["service"]["domain"] = $result;

                        Capsule::table('tblcustomfieldsvalues')
                            ->where('fieldid', $addon_custom_field_id)
                            ->where('relid', $params["addonId"])
                            ->where('value', '')
                            ->delete();
                        Capsule::table('tblcustomfieldsvalues')
                            ->updateOrInsert(
                                [
                                    'fieldid' => $addon_custom_field_id,
                                    'relid' =>  $params["addonId"],

                                ],
                                [
                                    'value' => $domain_name,
                                    'created_at' => Capsule::raw('CURRENT_TIMESTAMP'),
                                    'updated_at' => Capsule::raw('CURRENT_TIMESTAMP'),

                                ]
                            );
                    }
                    return $domain_name;
                }
            }
        }
    }
    return null;
}



/**
 * Upgrade an instance of service.
 * @param array $params common module parameters
 * @return string|array "success" or error message
 */
function gpf_ChangePackage(array $params)
{

    try {
        $key = sha1(trim($params['serveraccesshash']));
        $timestamp = str_shuffle(time());
        if (empty($params['customfields']['domain'])) {
            $domain = $params['customfields']['Domain'];
        } else {
            $domain = $params['customfields']['domain'];
        }

        $hash = sha1($key . $domain . $timestamp);
        $endpoint = 'packageupdate';
        $data = [
            'status' => 'active',
            'package' => $params['configoption1'],
            'domain' => $domain,
            'hash' => $hash,
            'timestamp' => $timestamp,
        ];

        $response = json_decode(curlRequest($endpoint, $params, 'POST', $data));

        if (substr($response->status, 0, 2) != 'OK') {
            $success = false;
            if (is_object($response)) {
                $errorMsg = $response->status;
            } else {
                $errorMsg = $response;
            }
        } else {
            $success = true;
            $errorMsg = '';
        }
    } catch (Exception $e) {
        logModuleCall(
            'gpf',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        $success = false;
        $errorMsg = $e->getMessage();
    }

    if ($success === false && $errorMsg == null) return 'Can not get remote response.';
    if ($success === false) return $errorMsg;
    return 'success';
}


/**
 * New instance of service.
 * @param array $params common module parameters
 * @return string|array "success" or error message
 */
function gpf_CreateAccount(array $params)
{
    if (function_exists('gpfaddon_CreateAccount')) {
        $gpaddon_response = gpfaddon_CreateAccount($params);
        if (is_array($gpaddon_response)) {
            if (isset($gpaddon_response['success']) && $gpaddon_response['success'] === false) {
                return $gpaddon_response['error'];
            }
            if (isset($gpaddon_response['success']) && $gpaddon_response['success'] === true) {
                gpf_ChangePackage($gpaddon_response['params']);
                return 'Addon upgrade processing in the background. Service cancelled: ' . $params["addonId"] . ' | Service upgrade ' . $gpaddon_response['serviceid'];
            }
        }
    }

    try {

        $key = sha1(trim($params['serveraccesshash']));
        $timestamp = str_shuffle(time());
        if (empty($params['customfields']['domain'])) {
            $domain = $params['customfields']['Domain'];
        } else {
            $domain = $params['customfields']['domain'];
        }

        if (empty($domain)) {
            $domain = gpf_FindDomain($params);
        }
        if (empty($domain)) {
            return 'Missing domain name';
        }

        try {
            Capsule::table('tblhosting')
                ->where('id', $params["serviceid"])
                ->update(
                    [
                        'domain' => $domain,
                    ]
                );
        } catch (\Exception $e) {
            echo "I couldn't update the value. {$e->getMessage()}";
        }
        $params['model']->serviceProperties->save(['domain' => $domain]);

        $hash = sha1($key . $domain . $timestamp);
        $password = gpfPassEncrypt($params['serveraccesshash'], $params['password']);
        $endpoint = 'autoregister';
        $data = [
            'package' => $params['configoption1'],
            'domain' => $domain,
            'password' => $password,
            'uemail' => $params['clientsdetails']['email'],
            'hash' => $hash,
            'timestamp' => $timestamp,
        ];

        $response = json_decode(curlRequest($endpoint, $params, 'POST', $data));
        if (substr($response->status, 0, 2) != 'OK') {
            $success = false;
            if (is_object($response)) {
                $errorMsg = $response->status;
            } else {
                $errorMsg = $response;
            }
        } else {
            $params['model']->serviceProperties->save(['gpftoken' => $response->token]);
            $success = true;
            $errorMsg = '';
            /*  $get_token_id_request = Capsule::table('tblproducts')
                ->join('tblcustomfields', 'tblcustomfields.relid', '=', 'tblproducts.id')
                ->select('tblcustomfields.id')
                ->where('tblproducts.configoption1', "=", $params['configoption1'])
                ->where('tblproducts.servertype', "=", 'gpf')
                ->where('tblcustomfields.fieldname', "=",  'gpftoken')
                ->first();

            $token_field_id = $get_token_id_request->id;
            if ($token_field_id != null) {

                $get_token_id_request = Capsule::table('tblcustomfieldsvalues')
                    ->updateOrInsert(
                        [
                            'fieldid' => $token_field_id,
                            'relid' => $params["serviceid"],
                        ],
                        [
                            'value' => $response->token,
                            'created_at' => Capsule::raw('CURRENT_TIMESTAMP'),
                            'updated_at' => Capsule::raw('CURRENT_TIMESTAMP'),
                        ]
                    );
            }
            */
        }
    } catch (Exception $e) {
        logModuleCall(
            'gpf',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        $success = false;
        $errorMsg = $e->getMessage();
    }
    if ($success === false && $errorMsg == null) return 'Can not get remote response.';
    if ($success === false) return $errorMsg;
    return 'success';
}

/**
 * Suspend an instance of service.
 * @param array $params common module parameters
 * @return string|array "success" or error message
 */
function gpf_SuspendAccount(array $params)
{
    try {
        $key = sha1(trim($params['serveraccesshash']));
        $timestamp = str_shuffle(time());
        if (empty($params['customfields']['domain'])) {
            $domain = $params['customfields']['Domain'];
        } else {
            $domain = $params['customfields']['domain'];
        }
        $hash = sha1($key . $domain . $timestamp);
        $endpoint = 'suspended';
        $data = [
            'domain' => $domain,
            'hash' => $hash,
            'timestamp' => $timestamp,
        ];

        $response = json_decode(curlRequest($endpoint, $params, 'POST', $data));

        if (substr($response->status, 0, 2) != 'OK') {
            $success = false;
            if (is_object($response)) {
                $errorMsg = $response->status;
            } else {
                $errorMsg = $response;
            }
        } else {
            $success = true;
            $errorMsg = '';
        }
    } catch (Exception $e) {
        logModuleCall(
            'gpf',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        $success = false;
        $errorMsg = $e->getMessage();
    }
    if ($success === false && $errorMsg == null) return 'Can not get remote response.';
    if ($success === false) return $errorMsg;
    return 'success';
}

/**
 * Un-suspend instance of service.
 * @param array $params common module parameters
 * @return string|array "success" or error message
 */
function gpf_UnsuspendAccount(array $params)
{
    try {
        $key = sha1(trim($params['serveraccesshash']));
        $timestamp = str_shuffle(time());
        if (empty($params['customfields']['domain'])) {
            $domain = $params['customfields']['Domain'];
        } else {
            $domain = $params['customfields']['domain'];
        }
        $hash = sha1($key . $domain . $timestamp);
        $endpoint = 'active';
        $data = [
            'domain' => $domain,
            'hash' => $hash,
            'timestamp' => $timestamp,
        ];

        $response = json_decode(curlRequest($endpoint, $params, 'POST', $data));

        if (substr($response->status, 0, 2) != 'OK') {
            $success = false;
            if (is_object($response)) {
                $errorMsg = $response->status;
            } else {
                $errorMsg = $response;
            }
        } else {
            $success = true;
            $errorMsg = '';
        }
    } catch (Exception $e) {
        logModuleCall(
            'gpf',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        $success = false;
        $errorMsg = $e->getMessage();
    }
    if ($success === false && $errorMsg == null) return 'Can not get remote response.';
    if ($success === false) return $errorMsg;
    return 'success';
}

/**
 * Terminate instance of service.
 * @param array $params common module parameters
 * @return string|array "success" or error message
 */
function gpf_TerminateAccount(array $params)
{
    try {
        $key = sha1(trim($params['serveraccesshash']));
        $timestamp = str_shuffle(time());
        if (empty($params['customfields']['domain'])) {
            $domain = $params['customfields']['Domain'];
        } else {
            $domain = $params['customfields']['domain'];
        }
        $hash = sha1($key . $domain . $timestamp);
        $endpoint = 'delete';
        $data = [
            'domain' => $domain,
            'hash' => $hash,
            'timestamp' => $timestamp,
        ];

        $response = json_decode(curlRequest($endpoint, $params, 'POST', $data));

        if (substr($response->status, 0, 2) != 'OK') {
            $success = false;
            if (is_object($response)) {
                $errorMsg = $response->status;
            } else {
                $errorMsg = $response;
            }
        } else {
            $success = true;
            $errorMsg = '';
        }
    } catch (Exception $e) {
        logModuleCall(
            'gpf',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        $success = false;
        $errorMsg = $e->getMessage();
    }
    if ($success === false && $errorMsg == null) return 'Can not get remote response.';
    if ($success === false) return $errorMsg;
    return 'success';
}

/**
 * Change the password for an instance of service.
 * @param array $params common module parameters
 * @return string|array "success" or error message
 */
function gpf_ChangePassword(array $params)
{
    try {
        $key = sha1(trim($params['serveraccesshash']));
        $timestamp = str_shuffle(time());
        if (empty($params['customfields']['domain'])) {
            $domain = $params['customfields']['Domain'];
        } else {
            $domain = $params['customfields']['domain'];
        }
        $hash = sha1($key . $domain . $timestamp);
        $password = gpfPassEncrypt($params['serveraccesshash'], $params['password']);
        $endpoint = 'passwordreset';
        $data = [
            'domain' => $domain,
            'password' => $password,
            'hash' => $hash,
            'timestamp' => $timestamp,
        ];

        $response = json_decode(curlRequest($endpoint, $params, 'POST', $data));

        if (substr($response->status, 0, 2) != 'OK') {
            $success = false;
            if (is_object($response)) {
                $errorMsg = $response->status;
            } else {
                $errorMsg = $response;
            }
        } else {
            // $params['model']->serviceProperties->save(['password' => $response->password]);
            $success = true;
            $errorMsg = '';
        }
    } catch (Exception $e) {
        logModuleCall(
            'gpf',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        $success = false;
        $errorMsg = $e->getMessage();
    }
    if ($success === false && $errorMsg == null) return 'Can not get remote response.';
    if ($success === false) return $errorMsg;
    return 'success';
}


/**
 * Renew an instance of service.
 * @param array $params common module parameters
 * @return string|array "success" or error message
 */
function gpf_Renew(array $params)
{
    try {
        $key = sha1(trim($params['serveraccesshash']));
        $timestamp = str_shuffle(time());
        if (empty($params['customfields']['domain'])) {
            $domain = $params['customfields']['Domain'];
        } else {
            $domain = $params['customfields']['domain'];
        }
        $hash = sha1($key . $domain . $timestamp);
        $endpoint = 'renew';
        $data = [
            'domain' => $domain,
            'hash' => $hash,
            'timestamp' => $timestamp,
        ];

        $response = json_decode(curlRequest($endpoint, $params, 'POST', $data));

        if (substr($response->status, 0, 2) != 'OK') {
            $success = false;
            if (is_object($response)) {
                $errorMsg = $response->status;
            } else {
                $errorMsg = $response;
            }
        } else {
            $success = true;
            $errorMsg = '';
        }
    } catch (Exception $e) {
        logModuleCall(
            'gpf',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        $success = false;
        $errorMsg = $e->getMessage();
    }
    if ($success === false && $errorMsg == null) return 'Can not get remote response.';
    if ($success === false) return $errorMsg;
    return 'success';
}



/**
 * Get status of an instance.
 * @param array $params common module parameters
 * @return string|array "success" or error message
 */
function gpf_Status(array $params)
{
    try {
        $key = sha1(trim($params['serveraccesshash']));
        $timestamp = str_shuffle(time());
        if (empty($params['customfields']['domain'])) {
            $domain = $params['customfields']['Domain'];
        } else {
            $domain = $params['customfields']['domain'];
        }
        $hash = sha1($key . $domain . $timestamp);
        $endpoint = 'list';
        $data = [
            'domain' => $domain,
            'hash' => $hash,
            'timestamp' => $timestamp,
        ];

        $response = json_decode(curlRequest($endpoint, $params, 'POST', $data));

        if (!isset($response->exist) || $response->exist != 'exists') {
            $success = false;
            if (is_object($response)) {
                $errorMsg = $response->status;
            } else {
                $errorMsg = $response;
            }
        } else {
            if ($response->status !== 'active') {
                $success = false;
                if (is_object($response)) {
                    $errorMsg = $response->status;
                } else {
                    $errorMsg = $response;
                }
            } else {

                $params['model']->serviceProperties->save(['gpftoken' => $response->token]);
                $success = true;
                if (is_object($response)) {
                    $errorMsg = $response->status . ' | Token: ' . $response->token;
                } else {
                    $errorMsg = $response;
                }
            }
        }
    } catch (Exception $e) {
        logModuleCall(
            'gpf',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        $success = false;
        $errorMsg = $e->getMessage();
    }
    if ($success === false && $errorMsg == null) return 'Can not get remote response.';
    if ($success === false) return array('error' => $errorMsg);
    return array('success' => $errorMsg);
}



/**
 * Displays a single sign-on link for a given instance of service.
 * @param array $params common module parameters
 * @return array returns a URL.
 */

function gpf_LoginLink(array $params)
{

    try {
        $key = sha1(trim($params['serveraccesshash']));
        $timestamp = str_shuffle(time());
        if (empty($params['customfields']['domain'])) {
            $domain = $params['customfields']['Domain'];
        } else {
            $domain = $params['customfields']['domain'];
        }
        $hash = sha1($key . $domain . $timestamp);
        $password = gpfPassEncrypt($params['serveraccesshash'], $params['password']);
        $hub = getTowerRs($hash, $domain, $timestamp)[1];
        $redirectUrl = 'https://' . $hub . '?action=autologin&domain=' . $domain . '&hash=' . $hash . '&password=' . $password . '&timestamp=' . $timestamp;


        return array(
            'success' => "Link to the service dashboard: " . $redirectUrl,
            //'redirectTo' => $redirectUrl,
            );
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'gpf',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        return array(
            'success' => false,
            'errorMsg' => $e->getMessage(),
        );
    }
}



/**
 * Client area output logic handling.
 * @param array $params common module parameters
 * @return array
 */
function gpf_ClientArea(array $params)
{

    // Determine the requested action and set service call parameters based on the action.
    $requestedAction = isset($_REQUEST['customAction']) ? $_REQUEST['customAction'] : '';

    $key = sha1(trim($params['serveraccesshash']));
    $timestamp = str_shuffle(time());
    if (empty($params['customfields']['domain'])) {
        $domain = $params['customfields']['Domain'];
    } else {
        $domain = $params['customfields']['domain'];
    }
    $hash = sha1($key . $domain . $timestamp);
    $password = gpfPassEncrypt($params['serveraccesshash'], $params['password']);

    $hub = getTowerRs($hash, $domain, $timestamp)[1];
    $hub_direct_link = 'https://' . $hub . '?action=autologin&domain=' . $domain . '&timestamp=' . $timestamp . '&hash=' . $hash . '&password=' . $password;
    $token = $params['customfields']['gpftoken'];

    if ($requestedAction == 'manage') {
        $templateFile = 'templates/manage.tpl';
    } else {
        $templateFile = 'templates/overview.tpl';
    }
    try {



        return array(
            'tabOverviewReplacementTemplate' => $templateFile,
            'templateVariables' => array(
                'gpfform' => $hub_direct_link,
                'gpftoken' => $token,
            ),
        );
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'gpf',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        // In an error condition, display the default overview.
        return array(
            'tabOverviewReplacementTemplate' => 'overview.tpl',
            'templateVariables' => array(
                'usefulErrorHelper' => $e->getMessage(),
            ),
        );
    }
}
